/*
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

/*
 * Name:    mt_usint.c
 *
 * Purpose: User interrupt handler functions for the MX Tcl GUI.
 */

#include <stdio.h>

#include <tcl.h>

#include "mxconfig.h"
#include "mx_util.h"
#include "mx_rec.h"

#include "mx_tcl.h"

#define MXGUI_USER_INTERRUPT_MAX_TCL_EVENTS	100

#define MXGUI_USER_INTERRUPT_VARIABLE		"Mx::user_interrupt"

static int Mxgui_user_interrupt_handler( void );

static Tcl_Interp *Mxgui_user_interrupt_handler_interp;

MX_EXPORT int
Mx_install_user_interrupt_handler_cmd( ClientData clientData,
			Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_install_user_interrupt_handler_cmd()";

	if ( objc != 1 ) {
		Tcl_WrongNumArgs( interp, 0, objv,
		"Usage: install_user_interrupt_handler" );
		return TCL_ERROR;
	}

	MX_DEBUG( 1,("%s invoked.", fname));

	mx_set_user_interrupt_function( Mxgui_user_interrupt_handler );

	Mxgui_user_interrupt_handler_interp = interp;

	return TCL_OK;
}

static int
Mxgui_user_interrupt_handler( void )
{
	const char fname[] = "Mxgui_user_interrupt_handler()";

	int user_interrupt, num_items;
	char *user_interrupt_ptr;

	MX_DEBUG( 1,("%s invoked.",fname));

	/* Process up to MXGUI_USER_INTERRUPT_MAX_TCL_EVENTS events from
	 * the Tcl event handler, so that the GUI part of the program does
	 * not freeze up.
	 */

	(void) mxtcl_process_tcl_events( MXGUI_USER_INTERRUPT_MAX_TCL_EVENTS );

	/***** Check to see if a user interrupt has been requested. *****/

	user_interrupt_ptr = Tcl_GetVar( Mxgui_user_interrupt_handler_interp,
				MXGUI_USER_INTERRUPT_VARIABLE, 0 );

	if ( user_interrupt_ptr == NULL ) {
		(void) mx_error( MXE_ILLEGAL_ARGUMENT, fname,
			"The variable '%s' does not exist.",
			MXGUI_USER_INTERRUPT_VARIABLE );

		return TRUE;
	}

	num_items = sscanf( user_interrupt_ptr, "%d", &user_interrupt );

	if ( num_items != 1 ) {
		(void) mx_error( MXE_TYPE_MISMATCH, fname,
		"The contents of the Tcl variable '%s' are not an integer.  "
		"Instead they are '%s'",
			MXGUI_USER_INTERRUPT_VARIABLE, user_interrupt_ptr );

		return TRUE;
	}

	/***** Set the user interrupt variable to zero. *****/

	(void) Tcl_SetVar( Mxgui_user_interrupt_handler_interp,
				MXGUI_USER_INTERRUPT_VARIABLE, "0", 0 );

	/***** Return the user interrupt status. *****/

	if ( user_interrupt ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

