#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package provide MxGui 0.23

###########################################################################
#                                                                         #
#  The following routine is the main entry point for the scan setup and   #
#  modification screens.                                                  #
#                                                                         #
###########################################################################

proc mxgui_scan_setup { record_list scan_name scan_type } {

	set scan_record [ Mx::object_name $scan_name ]

# Does this scan record already exist?

	set old_record_list [ itcl::find objects $scan_record ]

	set old_record_exists [ llength $old_record_list ]

# Dispatch scans the procedure for the requested scan type.

	switch -exact -- $scan_type {
		input_scan  -
		slit_scan   -
		motor_scan  { mxgui_motor_scan_setup $record_list \
			$scan_name $scan_type $scan_record $old_record_exists }

		2theta_scan { mxgui_2theta_scan_setup $record_list \
				$scan_name $scan_record $old_record_exists }

		list_scan   { mxgui_list_scan_setup $record_list \
				$scan_name $scan_record $old_record_exists }

		xafs_scan   { mxgui_xafs_scan_setup $record_list \
				$scan_name $scan_record $old_record_exists }

		default     { tk_dialog .unknown_scan_type_dialog \
			"Unknown scan type" \
			"Scan type '$scan_type' is not recognized." \
			error 0 "Done"

			    return }
	}
}

###########################################################################

proc mxgui_2theta_scan_setup { record_list scan_name \
				scan_record old_record_exists } {

	tk_dialog .unimplemented_scan_type_dialog "Unimplemented scan type" \
"Support within the MX GUI for scan type '2theta_scan' is not yet implemented."\
	error 0 "Done"
}

proc mxgui_list_scan_setup { record_list scan_name \
				scan_record old_record_exists } {

	tk_dialog .unimplemented_scan_type_dialog "Unimplemented scan type" \
"Support within the MX GUI for scan type 'list_scan' is not yet implemented."\
	error 0 "Done"
}

###########################################################################
#                                                                         #
#  The following procedure is the routine normally used to display all    #
#  the fields below the scan range area on the GUI.                       #
#                                                                         #
###########################################################################

proc mxgui_initialize_standard_scan_display { window record_list scan_record \
					scan_type old_record_exists \
					show_integration_time } {
#
# Put the input devices info and timer info in side by side frames.
#

	frame $window.readout_frame
	pack $window.readout_frame -padx 5 -pady 5
#
# First, initialize the input device display.
#
	set inputs_frame $window.readout_frame.inputs

	frame $inputs_frame

	mxgui_initialize_input_device_display $inputs_frame \
				$record_list $scan_record $old_record_exists

	frame $window.readout_frame.pad
	label $window.readout_frame.pad.text -text " " -width 10
	pack $window.readout_frame.pad.text

#
# Display the measurement info parameters.
#

	set measurement_frame $window.readout_frame.measurement

	frame $measurement_frame

	mxgui_scan_display_measurement_fields $measurement_frame \
					$scan_record $old_record_exists \
					$show_integration_time

	grid $inputs_frame $window.readout_frame.pad $measurement_frame

	frame $window.separator3 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator3 -expand yes -fill both

#
# Display datafile and plot type parameters.
#

	set datafile_plot_frame $window.datafile_plot_frame

	frame $datafile_plot_frame
	pack $datafile_plot_frame -padx 5 -pady 5

	mxgui_scan_display_datafile_and_plot_fields $datafile_plot_frame \
				$scan_record $scan_type $old_record_exists

	frame $window.separator4 -height 2 -borderwidth 1 -relief sunken
	pack $window.separator4 -expand yes -fill both

	return
}

###########################################################################
#                                                                         #
#  The following procedure sets up the initial display of the input       #
#  device fields.                                                         #
#                                                                         #
###########################################################################

proc mxgui_initialize_input_device_display { window \
			record_list scan_record old_record_exists } {

	set num_inputs_frame    $window.num_inputs_frame
	set input_device_frame  $window.input_device_frame

	frame $num_inputs_frame
	pack $num_inputs_frame

	if { $old_record_exists == 0 } {
		set num_input_devices 1
	} else {
		set num_input_devices \
			[ $scan_record get_field num_input_devices ]
	}

	iwidgets::entryfield $num_inputs_frame.num_input_devices \
				-labeltext "Number of input devices:"

	$num_inputs_frame.num_input_devices clear
	$num_inputs_frame.num_input_devices insert 0 $num_input_devices

	$num_inputs_frame.num_input_devices configure -validate \
					{ iwidgets::Entryfield::integer %P }

	pack $num_inputs_frame.num_input_devices -side left

#
# old_num_input_devices is meant to be a hidden field that stores the
# old number of input devices in a place the other procedures can find it.
# So don't make it visible by packing it or gridding it, etc.
#
	label $num_inputs_frame.old_num_input_devices -text 0

	frame $input_device_frame
	pack $input_device_frame

	label $input_device_frame.name -text "Input device name"

	grid $input_device_frame.name -column 1

	$num_inputs_frame.num_input_devices configure -command [ code \
		mxgui_scan_redo_input_device_fields \
			$input_device_frame $num_inputs_frame \
			$record_list $scan_record $old_record_exists ]

# Initialize the input device display.

	mxgui_scan_redo_input_device_fields \
			$input_device_frame $num_inputs_frame \
			$record_list $scan_record $old_record_exists

}

###########################################################################
#                                                                         #
#  The following routine causes the input device fields to be redisplayed #
#  after any change in the number of input devices for the scan.          #
#                                                                         #
###########################################################################

proc mxgui_scan_redo_input_device_fields { \
		input_device_frame num_inputs_frame \
		record_list scan_record old_record_exists } {

	set old_num_input_devices \
		[ $num_inputs_frame.old_num_input_devices cget -text ]

	set num_input_devices [ $num_inputs_frame.num_input_devices get ]

	if { $num_input_devices > $old_num_input_devices } {

		# Add new input devices to the list of input devices.

		for { set i $old_num_input_devices } \
				{ $i < $num_input_devices } { incr i } {

			set thisdevice $input_device_frame.device$i

			label ${thisdevice}_label -text "Device $i:"

			iwidgets::entryfield ${thisdevice}_name -command [ \
				code mxgui_scan_change_input_device \
				$record_list $thisdevice ]

			set arguments \$f\[$i\]

			label ${thisdevice}_plot -text $arguments

			grid ${thisdevice}_label ${thisdevice}_name \
				${thisdevice}_plot
		}
	} else {

		# Delete old input devices from the list.

		for { set i $num_input_devices } \
				{ $i < $old_num_input_devices } { incr i } {

			set thisdevice $input_device_frame.device$i

			destroy ${thisdevice}_label
			destroy ${thisdevice}_name
			destroy ${thisdevice}_plot
		}
	}

	$num_inputs_frame.old_num_input_devices configure \
				-text $num_input_devices
}

proc mxgui_scan_copy_input_device_fields_from_record { window \
						record_list scan_record } {

	set num_input_devices [ $scan_record get_field num_input_devices ]
	set input_device_list \
		[ split [ $scan_record get_field input_device_array ] ]

	set input_device_frame  $window.readout_frame.inputs.input_device_frame

	for { set i 0 } { $i < $num_input_devices } { incr i } {

		set thisdevice $input_device_frame.device$i

		${thisdevice}_name insert end [ lindex $input_device_list $i ]
	}
}

proc mxgui_scan_change_input_device { record_list thisdevice } {

	set device_name [ ${thisdevice}_name get ]

	if [ catch { $record_list get_record $device_name } device_record ] {
		tk_dialog .no_such_device_dialog "No Such Device" \
		"There is no record named '$device_name'" error 0 "OK"

		${thisdevice}_name clear

		return
	}
	set superclass [ $device_record get_field superclass ]

	switch -- $superclass {
		variable { return }
		device {}
		default {
			set t "Record '$device_name' is of superclass "
			append t "'$superclass' which is not allowed as "
			append t "an input device."

			tk_dialog .type_mismatch_dialog \
				"Illegal Record Superclass" $t error 0 "OK"

			${thisdevice}_name clear

			return
		}
	}

	set class [ $device_record get_field class ]

	switch -- $class {
		analog_input  -
		digital_input -
		motor         -
		scaler {}
		default {
			set t "Device '$device_name' may not be used as "
			append t "an input device.  Only scalers, motors, "
			append t "analog inputs, digital inputs, and "
			append t "variables may be used as input devices."

			tk_dialog .type_mismatch_dialog \
				"Illegal Record Class" $t error 0 "OK"

			${thisdevice}_name clear

			return
		}
	}
}

###########################################################################
#                                                                         #
#      The following routine displays measurement parameter fields.       #
#                                                                         #
###########################################################################

proc mxgui_scan_display_measurement_fields { measurement_frame scan_record \
				old_record_exists show_integration_time } {

	if { $show_integration_time == 0 } {
		set field_list [ list settling_time ]
	} else {
		set field_list [ list integration_time settling_time ]

		iwidgets::entryfield $measurement_frame.integration_time \
				-labeltext "Integration time:"

		$measurement_frame.integration_time insert 0 0
	}

	iwidgets::entryfield $measurement_frame.settling_time \
				-labeltext "Settling time:"

	$measurement_frame.settling_time insert 0 0

	if { $show_integration_time != 0 } {
		iwidgets::Labeledwidget::alignlabels \
				$measurement_frame.integration_time \
				$measurement_frame.settling_time
	}

	foreach widget $field_list {
		pack $measurement_frame.$widget -padx 10 -pady 5 \
						-fill x -expand yes
	}
}

proc mxgui_scan_copy_measurement_fields_from_record { window record_list \
					scan_record show_integration_time } {
	
	set measurement_frame $window.readout_frame.measurement

	if { $show_integration_time != 0 } {
		$measurement_frame.integration_time clear

		$measurement_frame.integration_time insert 0 \
			[ $scan_record get_field integration_time ]
	}

	$measurement_frame.settling_time clear

	$measurement_frame.settling_time insert end \
			[ $scan_record get_field settling_time ]
}

###########################################################################
#                                                                         #
#      The following routine displays datafile and plot fields.           #
#                                                                         #
###########################################################################

proc mxgui_scan_display_datafile_and_plot_fields { datafile_plot_frame \
				scan_record scan_type old_record_exists } {

	set datafile_frame $datafile_plot_frame.datafile_frame

	frame $datafile_frame
	pack $datafile_frame -side left 

	iwidgets::optionmenu $datafile_frame.type -labeltext "Datafile type:"

	$datafile_frame.type insert end SFF Text XAFS

	pack $datafile_frame.type

	iwidgets::entryfield $datafile_frame.name -labeltext "Datafile name:"

	pack $datafile_frame.name

	###################################################################

	label $datafile_plot_frame.pad -text " " -width 10
	pack $datafile_plot_frame.pad -side left -expand yes -fill x

	###################################################################

	set plotfile_frame $datafile_plot_frame.plotfile_frame

	frame $plotfile_frame
	pack $plotfile_frame -side left

	iwidgets::optionmenu $plotfile_frame.type -labeltext "Plotfile type:"

	$plotfile_frame.type insert end Gnuplot GnuXAFS None

	pack $plotfile_frame.type

	iwidgets::entryfield $plotfile_frame.arguments \
						-labeltext "Plot arguments:"

	pack $plotfile_frame.arguments

# Initialize the fields depending on the scan type.

	set datafile_name "junk.out"

	switch -- $scan_type {
		xafs_scan { 
			set datatype_selection XAFS
			set plotfile_selection GnuXAFS
			set plot_arguments "\$f\[0\]/\$f\[1\]"
		}
		default {
			set datatype_selection SFF
			set plotfile_selection Gnuplot
			set plot_arguments "\$f\[0\]"
		}
	}

	$datafile_frame.type select $datatype_selection

	$datafile_frame.name insert 0 $datafile_name

	$plotfile_frame.type select $plotfile_selection

	$plotfile_frame.arguments insert 0 $plot_arguments
}

proc mxgui_scan_copy_datafile_and_plot_fields_from_record { window \
						record_list scan_record } {

	set datafile_list [ split [ $scan_record get_field datafile ] ]
	set plot_list [ split [ $scan_record get_field plot ] ]

	set thisframe $window.datafile_plot_frame

# Handle the datafile fields.

	$thisframe.datafile_frame.name clear
	$thisframe.datafile_frame.name insert 0 [ lindex $datafile_list 1 ]

	switch -- [ lindex $datafile_list 0 ] {
		text { set selection Text }
		sff  { set selection SFF }
		xafs { set selection XAFS }
		default { set selection Unknown }
	}

	$thisframe.datafile_frame.type select $selection

# Handle the plot fields

	set plot_arguments [ lindex $plot_list 1 ]

	set plot_arguments [ string trim $plot_arguments \" ]

	$thisframe.plotfile_frame.arguments clear
	$thisframe.plotfile_frame.arguments insert 0 $plot_arguments

	switch -- [ lindex $plot_list 0 ] {
		gnuplot { set selection Gnuplot }
		gnuxafs { set selection GnuXAFS }
		none    { set selection None }
		default { set selection Unknown }
	}

	$thisframe.plotfile_frame.type select $selection
}

###########################################################################
#                                                                         #
# mxgui_perform_scan is used to run a scan and modify the top level gui   #
# display as necessary.                                                   #
#                                                                         #
###########################################################################

proc mxgui_perform_scan { scan_record } {

	set scan_name [ $scan_record cget -name ]

	set window .scan_control_window

	frame $window
	pack $window -before .message_log

	label $window.scan_label -text "Current scan: "

	label $window.scan_name -text $scan_name -font $MxGui::monospaced_font

	pack $window.scan_label $window.scan_name -side left

	frame $window.sep 
	pack $window.sep -side left -expand yes

	label $window.sep.junk
	pack $window.sep.junk

	button $window.abort_scan -background red -activebackground red \
		-text "Abort Scan" -command [ code mxgui_abort_scan ]

	pack $window.abort_scan -side right

	update

	catch { $scan_record perform_scan }

	update

	pack forget $window
	destroy $window

	update
}

proc mxgui_abort_scan {} {
	log_puts "Scan abort requested."

	set Mx::user_interrupt 1
}

